@@ -1,4 +1,5 @@
 {-# OPTIONS_GHC -fno-warn-missing-methods #-}
+{-# LANGUAGE CPP                   #-}
 {-# LANGUAGE DeriveFunctor         #-}
 {-# LANGUAGE FlexibleInstances     #-}
 {-# LANGUAGE MultiParamTypeClasses #-}
@@ -22,7 +23,6 @@ import Data.CaseInsensitive (mk)
 import Data.Char (isAlphaNum, isAscii, isLetter, isPrint, isSpace, ord)
 import qualified Data.Map as Map
 import Data.Maybe (catMaybes)
-import Data.Monoid
 import qualified Data.Set as Set
 import Data.Text (Text)
 import qualified Data.Text as Text
@@ -31,6 +31,9 @@ import Debug.Trace
 import Skylighting.Regex
 import Skylighting.Types
 import Text.Printf (printf)
+#if !MIN_VERSION_base(4,11,0)
+import Data.Semigroup
+#endif
 
 newtype ContextStack = ContextStack{ unContextStack :: [Context] }
   deriving (Show)
@@ -387,15 +390,15 @@ stringDetect caseSensitive s inp = do
 normalChunk :: TokenizerM Text
 normalChunk = do
   inp <- gets input
-  case BS.uncons inp of
+  case UTF8.uncons inp of
     Nothing -> mzero
     Just (c, _)
       | c == ' ' ->
         let bs = BS.takeWhile (==' ') inp
         in  takeChars (BS.length bs)
       | isAscii c && isAlphaNum c ->
-        let bs = BS.takeWhile isAlphaNum inp
-        in  takeChars (BS.length bs)
+        let (bs, _) = UTF8.span isAlphaNum inp
+        in  takeChars (UTF8.length bs)
       | otherwise -> takeChars 1
 
 includeRules :: Maybe TokenType -> ContextName -> ByteString
@@ -480,8 +483,8 @@ detectIdentifier :: ByteString -> TokenizerM Text
 detectIdentifier inp = do
   case BS.uncons inp of
     Just (c, t) | isLetter c || c == '_' ->
-      takeChars $ 1 + maybe 0 id (BS.findIndex
-                (\d -> not (isAlphaNum d || d == '_')) t)
+      takeChars $ 1 + maybe (BS.length t) id
+                (BS.findIndex (\d -> not (isAlphaNum d || d == '_')) t)
     _ -> mzero
 
 lineContinue :: ByteString -> TokenizerM Text
@@ -624,7 +627,7 @@ pCStringChar = do
   next <- A.anyChar
   case next of
        c | c == 'x' || c == 'X' -> () <$ A.takeWhile1 (A.inClass "0-9a-fA-F")
-         | c == '0' -> () <$ A.takeWhile1 (A.inClass "0-7")
+         | c == '0' -> () <$ A.takeWhile (A.inClass "0-7")
          | A.inClass "abefnrtv\"'?\\" c -> return ()
          | otherwise -> mzero
 
